#include "stdafx.h"
#include "CController.h"
#include "Debug.h"

#define INITIAL_GNAT_SEPARATION 50
#define SAVE_WEIGHTS
// #define BEST_OF_MULTIPLE_FILES

// these hold the geometry of the gnats and the fungi
const int	 NumGnatVerts = 4;
const SPoint gnat[NumGnatVerts] = {SPoint(0, 1),
                                         SPoint(-1, -1),
                                         SPoint(0, 0),
                                         SPoint(1, -1)};
										 // Then back to the start: SPoint(0, 1)};


const int NumFungusVerts = 4;
const SPoint fungus[NumFungusVerts] = {SPoint(-1, -1),
                                   SPoint(-1, 1),
                                   SPoint(1, 1),
                                   SPoint(1, -1)};



//---------------------------------------constructor---------------------
//
//	initilaize the gnats, their brains and the GA factory
//
//-----------------------------------------------------------------------
CController::CController(HWND hwndMain): m_NumGnats(CParams::iNumGnats), 
										                     m_pGA(NULL),
										                     m_bFastRender(false),
										                     m_iTicks(0),
										                     m_NumFungi(CParams::iNumFungi),
										                     m_hwndMain(hwndMain),
										                     m_iGenerations(0),
                                         cxClient(CParams::WindowWidth - 1),
                                         cyClient(CParams::WindowHeight - 1)
{
 int i;

// Fungus Quadrants
// **************
// Start upper left corner and go 3 rows across...
// Forbidden Zone runs between 170.33333... and 340.66666..., inclusivly.
// A fungus at 170 is 0.33333... away from the zone
// A fungus at 341 is 0.33333... away from the zone
// 0 and 511 are forbidden as well, the outer walls
// So we'll say the Forbidden Zone runs between 170 and 340 inclusivly.
// The following values are what get sent to RandInt(minValue, maxValue);
// Each fungus quad is 170 pixels wide and high,
// when we exclude pixel 0 and 511 where we do not want to place fungi.
// 1 thru 170,      171 thru 340,    341 thru 510
// Rem: Since these values are inclusive, total pixels w or h = (right - left) + 1

	// Row across the top...
	i = 0;
	rFungusQuads[i].left = 1;
	rFungusQuads[i].top  = 1;
	rFungusQuads[i].right  = 170;
	rFungusQuads[i++].bottom = 170; 

	rFungusQuads[i].left = 171;
	rFungusQuads[i].top  = 1;
	rFungusQuads[i].right  = 340;
	rFungusQuads[i++].bottom = 170;

	rFungusQuads[i].left = 341;
	rFungusQuads[i].top  = 1;
	rFungusQuads[i].right  = 510;
	rFungusQuads[i++].bottom = 170;

	// Middle row...
	rFungusQuads[i].left = 1;
	rFungusQuads[i].top  = 171;
	rFungusQuads[i].right  = 170;
	rFungusQuads[i++].bottom = 340; 

	// Forbidden zone...
	// rFungusQuads[i].left = 171;
	// rFungusQuads[i].top  = 171;
	// rFungusQuads[i].right  = 340;
	// rFungusQuads[i++].bottom = 340;

	rFungusQuads[i].left = 341;
	rFungusQuads[i].top  = 171;
	rFungusQuads[i].right  = 510;
	rFungusQuads[i++].bottom = 340;


	// Bottom row...
	rFungusQuads[i].left = 1;
	rFungusQuads[i].top  = 341;
	rFungusQuads[i].right  = 170;
	rFungusQuads[i++].bottom = 510; 

	rFungusQuads[i].left = 171;
	rFungusQuads[i].top  = 341;
	rFungusQuads[i].right  = 340;
	rFungusQuads[i++].bottom = 510;

	rFungusQuads[i].left = 341;
	rFungusQuads[i].top  = 341;
	rFungusQuads[i].right  = 510;
	rFungusQuads[i].bottom = 510;
	m_curQuad = 0;

	for(i = 0; i < m_NumGnats; ++i)
	{
		SVector2D *pNew = new SVector2D(0, 0);
	    m_vecStartPositions.push_back(*pNew);
	}
    RandomizeGnatStartPositions();
	// let's create the gnats
	for(i = 0; i < m_NumGnats; ++i)
	{
		m_vecGnats.push_back(CNeuralGnat(m_vecStartPositions[i], i)); // was this, but I don't understand
        m_vecStartPositionIdexes.push_back(i);
	}
 
	// get the total number of weights used in the gnats
	// NN so we can initialise the GA
	m_NumWeightsInNN = m_vecGnats[0].GetNumberOfWeights();

	// initialize the Genetic Algorithm class
	m_pGA = new CGenAlg(m_NumGnats,
                        CParams::dMutationRate,
	                    CParams::dCrossoverRate,
	                    m_NumWeightsInNN);

	// Get the weights from the GA and insert into the gnats brains
	m_vecThePopulation = m_pGA->GetChromos();

    LoadAllWeights();

	for(i = 0; i < m_NumGnats; i++)
		m_vecGnats[i].PutWeights(m_vecThePopulation[i].vecWeights);

	// initialize fungi in random positions within the application window

	for(i = 0; i < m_NumFungi; ++i)
	{
		int x, y;
		GetRandomFungusPosition(&x, &y, i % NUM_QUADS);
	 	m_vecFungi.push_back(SVector2D(x, y));
	}

	//create a pen for the graph drawing
	m_BluePen  = CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
	m_RedPen   = CreatePen(PS_SOLID, 1, RGB(255, 0, 0));
	m_GreenPen = CreatePen(PS_SOLID, 1, RGB(0, 150, 0));
	m_YellowPen = CreatePen(PS_SOLID, 1, RGB(255, 255, 0));
	m_OldPen	= NULL;

	// fill the vertex buffers
	for (i = 0; i < NumGnatVerts; ++i)
	{
		m_GnatVB.push_back(gnat[i]);
	}

	for (i = 0; i < NumFungusVerts; ++i)
	{
		m_FungusVB.push_back(fungus[i]);
	}


    // If "Elite" was nothing special, we should have   (double)m_iGenerations * (double)iNumElite / (double)iNumGnats elite wins
    m_ExpectedEliteWinsRatio = (double)CParams::iNumElite / (double)CParams::iNumGnats;
	m_iPrevGenerations = -1;
    m_EliteWins = 0;
	m_BestComposite = 0;
	m_Num_Saves = 0;
	m_Num_Loads = 0;
    m_dAvgGeneticDifference = 0;

	bUpdated = false;
	bSaveAllWeights = false;
	bSaveWeights = false;
	bLoadWeights = false;
	bLoadAllWeights = false;
	bShowVectors = false;
	m_dScalefactor = (double)CParams::iNumTicks / 2048.0;
}


//--------------------------------------destructor-------------------------------------
//
//--------------------------------------------------------------------------------------
CController::~CController()
{
  if(m_pGA)
  {
     delete	m_pGA;
  }

	DeleteObject(m_BluePen);
	DeleteObject(m_RedPen);
	DeleteObject(m_GreenPen);
	DeleteObject(m_YellowPen);
}


//---------------------WorldTransform--------------------------------
//
//	sets up the translation matrices for the fungi and applies the
//	world transform to each vertex in the vertex buffer passed to this
//	method.
//-------------------------------------------------------------------
void CController::WorldTransform(vector<SPoint> &VBuffer, SVector2D vPos)
{
	//create the world transformation matrix
	C2DMatrix matTransform;
	
	//scale
	matTransform.Scale(CParams::dFungusScale, CParams::dFungusScale);
	
	//translate
	matTransform.Translate(vPos.x, vPos.y);

	//transform the ships vertices
	matTransform.TransformSPoints(VBuffer);
}

bool CController::FungusIsCloseToGnat(SVector2D &vFungusPos)
{
 SVector2D	vGnatPos(0, 0);
 for(int i = 0; i < m_NumGnats; ++i)
 {
	 vGnatPos = m_vecGnats[i].Position() - vFungusPos;
	 if(Vec2DLength(vGnatPos) <= 10.0)return true;
 }
 return false;
}


void CController::GetRandomFungusPosition(int *pX, int *pY, int quad)
{
 SVector2D	vFungusPos(0, 0);
 
 vFungusPos.x = RandInt(rFungusQuads[quad].left, rFungusQuads[quad].right);
 vFungusPos.y = RandInt(rFungusQuads[quad].top, rFungusQuads[quad].bottom);

 // Did this fungus land near a gnat?
 while(FungusIsCloseToGnat(vFungusPos))
 {
       vFungusPos.x = RandInt(rFungusQuads[quad].left, rFungusQuads[quad].right);
       vFungusPos.y = RandInt(rFungusQuads[quad].top, rFungusQuads[quad].bottom);
 }
 *pX = (int)vFungusPos.x;
 *pY = (int)vFungusPos.y;
}

bool CController::GnatIsCloseToOthers(SVector2D &vGnatPos, int curGnat)
{
 SVector2D	vOtherPos(0, 0);
 if(!curGnat)return false;
 for(int i = 0; i < curGnat; ++i)
 {
	 vOtherPos = m_vecStartPositions[i] - vGnatPos;
	 if(Vec2DLength(vOtherPos) <= INITIAL_GNAT_SEPARATION)return true;
 }
 return false;
}

void CController::GetRandomGnatPosition(int *pX, int *pY, int quad, int iGnat)
{
 SVector2D	vGnatPos(0, 0);
 vGnatPos.x = RandInt(rFungusQuads[quad].left + 5, rFungusQuads[quad].right - 5);
 vGnatPos.y = RandInt(rFungusQuads[quad].top + 5, rFungusQuads[quad].bottom - 5);

 // Did this gnat land near another gnat?
 while(GnatIsCloseToOthers(vGnatPos, iGnat))
 {
       vGnatPos.x = RandInt(rFungusQuads[quad].left + 5, rFungusQuads[quad].right - 5);
       vGnatPos.y = RandInt(rFungusQuads[quad].top + 5, rFungusQuads[quad].bottom - 5);
 }
 *pX = (int)vGnatPos.x;
 *pY = (int)vGnatPos.y;
}

void CController::RandomizeGnatStartPositions(void)
{
 int x, y;

 for(int n = 0; n < m_NumGnats; n++)
 {
     GetRandomGnatPosition(&x, &y, n % NUM_QUADS, n);
	 m_vecStartPositions[n].x = x;
	 m_vecStartPositions[n].y = y;
 }
}



//-------------------------------------Update-----------------------------
//
//	This is the main workhorse. The entire simulation is controlled from here.
//
//	The comments should explain what is going on adequately.
//-------------------------------------------------------------------------

bool CController::Update()
{
 // Run the gnats through CParams::iNumTicks amount of cycles. During
 // this loop each gnats NN is constantly updated with the appropriate
 // information from its surroundings. The output from the NN is obtained
 // and the gnat is moved. If it encounters a fungus its fitness is
 // updated appropriately,

 if(m_iTicks++ < CParams::iNumTicks)
 {
	for(int currentGnat = 0; currentGnat < m_NumGnats; currentGnat++)
 	{
		// update the NN and position
        if(!m_vecGnats[currentGnat].Update(m_vecFungi, m_vecGnats, !m_bFastRender))
        {
           // error in processing the neural net
           MessageBox(m_hwndMain, "Wrong amount of NN inputs!", "Error", MB_OK);
		   return false;
        }
				
		// see if it's found a fungus
		double distanceToNeighbour;
        int nFungusIndex = m_vecGnats[currentGnat].CheckForFungus(m_vecFungi, &distanceToNeighbour);
		if(nFungusIndex >= 0)
        {
           // we have discovered a fungus so increase fitness
		   // but if(distanceToNeighbour <= 20.0) no soup for you!
		   if(distanceToNeighbour > 20.0)
		   {
              m_vecGnats[currentGnat].IncrementFitness();
		   }

           // fungus found so replace the fungus with another at a random position
		   int x, y;
		   GetRandomFungusPosition(&x, &y, m_curQuad);
		   if(++m_curQuad == NUM_QUADS)m_curQuad = 0;
		   m_vecFungi[nFungusIndex].x = x;
		   m_vecFungi[nFungusIndex].y = y;
		   // update the chromos fitness score
		   m_vecThePopulation[currentGnat].dFitness = m_vecGnats[currentGnat].Fitness();
        }
	}
	return true;
 }

 // Another generation has been completed.
 // Time to run the GA and update the gnats with their new NNs

 // reset cycles
 m_iTicks = 0;

//****************************************


 // run the GA to create a new population
 // m_vecThePopulation = m_pGA->Epoch(m_vecThePopulation); was here

 // Update the stats to be used in our stat window
 // Rank & sort - worst to best order...
 // This passes the population to the GA,
 // who will employ the latest population passed.
 // After this call m_vecThePopulation here is redundant,
 // and only employed here if we are exporting the weights
 // with a call to m_pGA->OrderForExport(m_vecThePopulation)
 // When Epoch is finally called, m_vecThePopulation will be
 // replaced by the newly evolved population.
 m_pGA->SortAndRank(m_vecThePopulation); // sorted worst to best order

 // The best of previous round (the "elites") were placed in the front of the array,
 // at index 0 thru (CParams::iNumElite - 1)
 // Did one of them win again?
 if(m_pGA->Best() < CParams::iNumElite)
 {
	++m_EliteWins;
 }

 dBestWorstFitnessRatio = 0;
 if(m_pGA->WorstFitness() > 0)
 {
    dBestWorstFitnessRatio = m_pGA->BestFitness() / m_pGA->WorstFitness();
 }
 dBestFitness = 3.0 * m_pGA->BestFitness();
 double avgFitness = 3.0 * m_pGA->AverageFitness(); 
 dBestFitness /= m_dScalefactor; // 4.0 for 8192
 avgFitness /= m_dScalefactor; // 4.0; 

 if(m_iGenerations)
 {
    m_dBestFitnessRollingAvg = (dBestFitness / 2.0) + (m_vecRollingBestFitness[m_iGenerations - 1] / 2.0);
    m_dAvgFitnessRollingAvg = (avgFitness / 2.0) + (m_vecRollingAvFitness[m_iGenerations - 1] / 2.0);
 }else
 {
    m_dBestFitnessRollingAvg = dBestFitness;
    m_dAvgFitnessRollingAvg = avgFitness;
 }
 m_vecRollingBestFitness.push_back(m_dBestFitnessRollingAvg);
 m_vecRollingAvFitness.push_back(m_dAvgFitnessRollingAvg);


 m_dBestAvgRollingRatio = (m_dAvgFitnessRollingAvg > 0) ? (m_dBestFitnessRollingAvg / m_dAvgFitnessRollingAvg) : 0;
 m_Composite = ((m_dAvgFitnessRollingAvg * 2) + (m_dBestFitnessRollingAvg / 2));

 if(m_Composite > m_BestComposite)
 {
    m_BestComposite = m_Composite;
    DebugMessage("New BestComposite");
    DebugValue("Generations", m_iGenerations);
	DebugFloatValue("BestComposite", m_BestComposite, 5);
	DebugFloatValue("dBestFitnessRollingAvg", m_dBestFitnessRollingAvg, 5);
	DebugFloatValue("dAvgFitnessRollingAvg", m_dAvgFitnessRollingAvg, 5);
	DebugFloatValue("dBestAvgRollingRatio", m_dBestAvgRollingRatio, 5);
	if(m_iGenerations)
	{
       double ratio = 100.0 * (((double)m_EliteWins / (double)m_iGenerations) - m_ExpectedEliteWinsRatio);
	   DebugFloatValue("Best win percent", ratio, 5);
	}
	DebugParams("Best index", "Fitness", m_pGA->Best(), (int)m_vecGnats[m_pGA->Best()].Fitness());
	DebugParams("Worst index", "Fitness", m_pGA->Worst(), (int)m_vecGnats[m_pGA->Worst()].Fitness());
	DebugFloatValue("BestWorstFitnessRatio", dBestWorstFitnessRatio, 5);
    DebugMessage("");

#ifdef SAVE_WEIGHTS
    if(m_Composite > 125.0)
    {
       bSaveAllWeights = true;
    }
#endif
 }

 // increment the generation counter
 ++m_iGenerations;

 if(bSaveAllWeights || bSaveWeights)
 {
    // The following call expects m_pGA->SortAndRank() was previously done on m_vecThePopulation
    m_vecThePopulation = m_pGA->OrderForExport(m_vecThePopulation);
 }

 if(bSaveAllWeights)
 {
    SaveAllWeights();
    bSaveAllWeights = false;
	++m_Num_Saves;
 }

 if(bSaveWeights)
 {
    SaveBestWeights(CParams::iNumElite - 1);
    bSaveWeights = false;
	++m_Num_Saves;
 }

 bUpdated = false;

 if(bLoadAllWeights)
 {
    LoadAllWeights();
	bLoadAllWeights = false;
 }else
 {
    // run the GA to create a new population
    // The following call expects m_pGA->SortAndRank() was previously done on m_vecThePopulation
    m_vecThePopulation = m_pGA->Epoch();
 }

 if(bLoadWeights)
 {
    LoadBestWeights(0);
    bLoadWeights = false;
 }


 // Insert the new (hopefully) improved brains back into the gnats
 // and reset their positions etc
 int i;
 RandomizeGnatStartPositions();
 RandomizeStartPositionIndexes();
 for(i = 0; i < m_NumGnats; ++i)
 {
	 m_vecGnats[i].PutWeights(m_vecThePopulation[i].vecWeights);
 	 m_vecGnats[i].Reset(m_vecStartPositions[m_vecStartPositionIdexes[i]]);
 }

 for(i = 0; i < m_NumFungi; ++i)
 {
	 int x, y;
	 GetRandomFungusPosition(&x, &y, i % NUM_QUADS);
	 m_vecFungi[i].x = x;
	 m_vecFungi[i].y = y;
 }

 return true;
}



bool CController::NeedUpdate()
{
	return (m_bFastRender) ? !bUpdated : true;
}

void CController::DrawGnat(HDC surface, int gnatIndex)
{
 int iHandedness = m_vecGnats[gnatIndex].GetHandedness();

 if(iHandedness == RIGHTHANDED)
 {
    SelectObject(surface, m_YellowPen);
 }else
 {
	if(gnatIndex >= CParams::iNumElite)
    {
       SelectObject(surface, m_OldPen);
	}else
	{
       // we want the fittest displayed in red
       SelectObject(surface, m_RedPen);
	}
 }

 // Grab the gnat vertices
 vector<SPoint> gnatVB = m_GnatVB;
        
 // Transform the vertex buffer
 m_vecGnats[gnatIndex].WorldTransform(gnatVB, 2.0);

 // Draw it
 MoveToEx(surface, (int)gnatVB[0].x, (int)gnatVB[0].y, NULL);
 for(int vertIndex = 1; vertIndex < NumGnatVerts; vertIndex++)
 {
     LineTo(surface, (int)gnatVB[vertIndex].x, (int)gnatVB[vertIndex].y);
 }
 LineTo(surface, (int)gnatVB[0].x, (int)gnatVB[0].y);


 gnatVB = m_GnatVB;

 m_vecGnats[gnatIndex].WorldTransform(gnatVB, 4.0);

 // Draw it
 MoveToEx(surface, (int)gnatVB[0].x, (int)gnatVB[0].y, NULL);
 for(int vertIndex = 1; vertIndex < NumGnatVerts; vertIndex++)
 {
     LineTo(surface, (int)gnatVB[vertIndex].x, (int)gnatVB[vertIndex].y);
 }
 LineTo(surface, (int)gnatVB[0].x, (int)gnatVB[0].y);


 if(gnatIndex >= CParams::iNumElite)
 {
    SelectObject(surface, m_OldPen);
 }else
 {
    // we want the fittest displayed in red
    SelectObject(surface, m_RedPen);
 }

 gnatVB = m_GnatVB;
        
 // Transform the vertex buffer
 m_vecGnats[gnatIndex].WorldTransform(gnatVB, 6.0);

 // Draw it
 MoveToEx(surface, (int)gnatVB[0].x, (int)gnatVB[0].y, NULL);
 for(int vertIndex = 1; vertIndex < NumGnatVerts; vertIndex++)
 {
     LineTo(surface, (int)gnatVB[vertIndex].x, (int)gnatVB[vertIndex].y);
 }
 LineTo(surface, (int)gnatVB[0].x, (int)gnatVB[0].y);
}


void CController::DrawFungi(HDC surface)
{
 for(int i = 0; i < m_NumFungi; ++i)
 {
	 // grab the vertices for the fungus shape
	 vector<SPoint> fungusVB = m_FungusVB;
 	 WorldTransform(fungusVB, m_vecFungi[i]);
	 MoveToEx(surface, (int)fungusVB[0].x, (int)fungusVB[0].y, NULL);
	 for(unsigned vert = 1; vert < fungusVB.size(); vert++)
 	 {
		 LineTo(surface, (int)fungusVB[vert].x, (int)fungusVB[vert].y);
	 }
 	 LineTo(surface, (int)fungusVB[0].x, (int)fungusVB[0].y);
 }
}


//------------------------------------Render()--------------------------------------
//
//----------------------------------------------------------------------------------
void CController::Render(HDC surface)
{
 // render the stats
 string s;

 if(!m_bFastRender || (m_iGenerations != m_iPrevGenerations))
 {
    if(m_Num_Saves)
    {
	   s = "Generation: " + itos(m_iGenerations) + ",  Num. Saves: " + itos(m_Num_Saves);
    }else
    {
 	   s = "Generation: " + itos(m_iGenerations);
    }

    TextOut(surface, 5, 0, s.c_str(), s.size());
    m_iPrevGenerations = m_iGenerations;
 }

 // do not render if running at accelerated speed
 if(!m_bFastRender)
 {
	bUpdated = true;

	// keep a record of the old pen
    m_OldPen = (HPEN)SelectObject(surface, m_GreenPen);

    // Forbidden zone 171 thru 340, inclusive
    // The LineTo function draws a line from the current position up to, but not including, the specified point. 

	MoveToEx(surface, 171, 171, NULL);
	LineTo(surface, 341, 171);

	MoveToEx(surface, 171, 340, NULL);
	LineTo(surface, 341, 340);

	MoveToEx(surface, 171, 171, NULL);
	LineTo(surface, 171, 341);

	MoveToEx(surface, 340, 171, NULL);
	LineTo(surface, 340, 341);

    // render the fungi
	DrawFungi(surface);
       		
    // we want the fittest displayed in red
    SelectObject(surface, m_RedPen);

	// render the gnats
 	for(int i = 0; i < m_NumGnats; i++)
	{
	    DrawGnat(surface, i);
	} // end for()

	if(bShowVectors)
	{
	   SVector2D vGnatPos(0, 0);
	   int xClosestGnat = 0;
	   int yClosestGnat = 0;
	   int xFungus = 0;
	   int yFungus = 0;

       m_vecGnats[(CParams::iNumElite - 1)].GetDisplayedPoints(&xClosestGnat, &yClosestGnat, &xFungus, &yFungus);
       vGnatPos = m_vecGnats[(CParams::iNumElite - 1)].Position();
       if((xClosestGnat >= 0) && (xClosestGnat <= 511) && (yClosestGnat >= 0) && (yClosestGnat <= 511))
       {
	       MoveToEx(surface, (int)vGnatPos.x, (int)vGnatPos.y, NULL);
	       LineTo(surface, xClosestGnat, yClosestGnat);
       }

       if(xFungus > -1)
       {
          if((xFungus >= 0) && (xFungus <= 511) && (yFungus >= 0) && (yFungus <= 511))
          {
	          MoveToEx(surface, (int)vGnatPos.x, (int)vGnatPos.y, NULL);
			  LineTo(surface, xFungus, yFungus);
          }
       }
	} // end if(bShowVectors)

    // Put the old pen back
    SelectObject(surface, m_OldPen);
 }else
 {
   if(!bUpdated)
   {
      PlotStats(surface);
      bUpdated = true;
   }
 }

}
//--------------------------PlotStats-------------------------------------
//
//  Given a surface to draw on this function displays stats and a crude
//  graph showing best and average fitness
//------------------------------------------------------------------------
void CController::PlotStats(HDC surface)
{
 double ratio;
 string s;

 if(m_iGenerations)
 { 
    ratio = 100.0 * (((double)m_EliteWins / (double)m_iGenerations) - m_ExpectedEliteWinsRatio);
	s = "Best win percent: " + ftos((float)ratio);
    TextOut(surface, 5, 20, s.c_str(), s.size());
 }

  s = "Best Score: " + ftos((float)m_dBestFitnessRollingAvg) +
	  ",  Average Score: " + ftos((float)m_dAvgFitnessRollingAvg) +
	  ",  Best/Avg Ratio: " + ftos((float)m_dBestAvgRollingRatio);
  TextOut(surface, 5, 40, s.c_str(), s.size());
  s = "Composite Score: " + ftos((float)m_Composite) + ",  Best Composite Score: " + ftos((float)m_BestComposite);
  TextOut(surface, 5, 60, s.c_str(), s.size());

  // render the graph
  float HSlice;
  float VSlice;
  float x = 0;
  unsigned int i;
  HSlice = (float)cxClient/(m_iGenerations+1);
  VSlice = 1.5f;

    // plot the graph for the average fitness
    x = 0;

    m_OldPen = (HPEN)SelectObject(surface, m_BluePen);

    MoveToEx(surface, 0, cyClient, NULL);
    
    for (i=0; i < m_vecRollingAvFitness.size(); ++i)
    {
       LineTo(surface, (int)x, (int)floor((cyClient - VSlice * m_vecRollingAvFitness[i])));

       x += HSlice;
    }

    x = 0;

    // plot the graph for the best fitness
    SelectObject(surface, m_RedPen);


    MoveToEx(surface, 0, cyClient, NULL);

    for(i = 0; i < m_vecRollingBestFitness.size(); ++i)
    {
		// double dComposite = ((m_vecRollingAvFitness[i] * 2) + (m_vecRollingBestFitness[i] / 2));
        // LineTo(surface, (int)floor(x), cyClient - (int)floor(VSlice * dComposite));
        LineTo(surface, (int)floor(x), cyClient - (int)floor(VSlice * m_vecRollingBestFitness[i]));
        x += HSlice;
    }

    //replace the old pen
    SelectObject(surface, m_OldPen);
}

void CController::SaveBestWeights(int index)
{
 FILE *fp = fopen("Weights.bin", "wb");
 if(fp == NULL)return;
 double *pWeights = new double [m_NumWeightsInNN];
 for(int i = 0; i < m_NumWeightsInNN; i++)
 {
	 pWeights[i] = m_vecThePopulation[index].vecWeights[i];
 }
 fwrite(pWeights, sizeof(double), m_NumWeightsInNN, fp);
 delete [] pWeights;
 fclose(fp);
}

void CController::SaveAllWeights(void)
{
 FILE *fp = fopen("AllWeights.bin", "wb");
 if(fp == NULL)return;
 double *pWeights = new double [m_NumWeightsInNN * m_NumGnats];
 int n = 0;
 for(int i = 0; i < m_NumGnats; i++)
 {
     for(int j = 0; j < m_NumWeightsInNN; j++)
     {
	     pWeights[n++] = m_vecThePopulation[i].vecWeights[j];
     }
 }
 fwrite(pWeights, sizeof(double), m_NumWeightsInNN * m_NumGnats, fp);
 delete [] pWeights;
 fclose(fp);
}

void CController::LoadBestWeights(int index)
{
 FILE *fp = fopen("Weights.bin", "rb");
 if(fp == NULL)return;

 double *pWeights = new double [m_NumWeightsInNN];
 ZeroMemory(pWeights, sizeof(double) * m_NumWeightsInNN);
 fread(pWeights, sizeof(double), m_NumWeightsInNN, fp);
 for(int i = 0; i < m_NumWeightsInNN; i++)
 {
	 m_vecThePopulation[index].vecWeights[i] = pWeights[i];
 }
 delete [] pWeights;
 fclose(fp);
}

#ifdef BEST_OF_MULTIPLE_FILES

void CController::LoadAllWeights(void)
{
 FILE *fp;
 double *p01Weights;
 double *p02Weights;
 int totWeights = m_NumWeightsInNN * m_NumGnats;

 fp = fopen("01Weights.bin", "rb");
 if(fp == NULL)return;
 p01Weights = new double [totWeights];
 ZeroMemory(p01Weights, sizeof(double) * totWeights);
 fread(p01Weights, sizeof(double), totWeights, fp);
 fclose(fp);

 fp = fopen("02Weights.bin", "rb");
 if(fp == NULL)return;
 p02Weights = new double [totWeights];
 ZeroMemory(p02Weights, sizeof(double) * totWeights);
 fread(p02Weights, sizeof(double), totWeights, fp);
 fclose(fp);


 int i, n = 0;
 for(i = 0; i < 16; i++)
 {
     for(int j = 0; j < m_NumWeightsInNN; j++)
     {
	     m_vecThePopulation[i].vecWeights[j] = p01Weights[n++];
     }
 }

 n = 0;
 for(; i < 32; i++)
 {
     for(int j = 0; j < m_NumWeightsInNN; j++)
     {
	     m_vecThePopulation[i].vecWeights[j] = p02Weights[n++];
     }
 }

 delete [] p01Weights;
 delete [] p02Weights;
}

#else

void CController::LoadAllWeights(void)
{
 FILE *fp = fopen("AllWeights.bin", "rb");
 if(fp == NULL)return;
 int totWeights = m_NumWeightsInNN * m_NumGnats;
 double *pWeights = new double [totWeights];
 ZeroMemory(pWeights, sizeof(double) * totWeights);
 fread(pWeights, sizeof(double), totWeights, fp);
 int n = 0;
 for(int i = 0; i < m_NumGnats; i++)
 {
     for(int j = 0; j < m_NumWeightsInNN; j++)
     {
	     m_vecThePopulation[i].vecWeights[j] = pWeights[n++];
     }
 }

 delete [] pWeights;
 fclose(fp);
}

#endif


void CController::RandomizeStartPositionIndexes(void)
{
 for(int n = 0; n < NUM_QUADS; n++)
 {
     for(int i = 0; i < m_NumGnats; i++)
     {
	     int rand = RandInt(0, m_NumGnats - 1);
	     while(rand == i)
	     {
		       rand = RandInt(0, m_NumGnats - 1);
	     }

	     int temp =  m_vecStartPositionIdexes[i];
	     m_vecStartPositionIdexes[i] = m_vecStartPositionIdexes[rand];
	     m_vecStartPositionIdexes[rand] = temp;
     }
 }
}
